<?php
/**
 * The config file of Yidou.
 *
 * @copyright   Copyright 2014-2015 青岛亿斗网络信息有限公司(QingDao Yidou Network Infomation Co,LTD, www.yidou.com.cn)
 * @license     business(商业软件)
 * @author      Chunsheng Wang <chunsheng@cnezsoft.com>
 * @package     yidou
 * @version     $Id$
 * @link        http://www.yidou.com.cn
 */
/* Judge class config and function getWebRoot exists or not, make sure php shells can work. */
if(!class_exists('config')){class config{}}
if(!function_exists('getWebRoot')){function getWebRoot(){}}

/* The basic settings. */
$config = new config();
$config->version      = '1.1';             // The version number, don't change.
$config->encoding     = 'UTF-8';           // The encoding.
$config->cookiePath   = '/';               // The path of cookies.
$config->webRoot      = getWebRoot();      // The web root.
$config->cookieLife   = time() + 2592000;  // The lifetime of cookies.
$config->timezone     = 'Asia/Shanghai';   // Time zone setting, more plese visit http://www.php.net/manual/en/timezones.php
$config->multi        = false;             // The config of multi site.
$config->checkVersion = true;              // Auto check for new version or not.

/* The request settins. */
$config->requestType = 'PATH_INFO';       // PATH_INFO or GET.
$config->pathType    = 'clean';           // If the request type is PATH_INFO, the path type.
$config->seoMode     = true;              // Whether turn on seo mode or not.
$config->requestFix  = '-';               // RequestType=PATH_INFO: the divider of the params, can be - _ or /
$config->moduleVar   = 'm';               // RequestType=GET: the name of the module var.
$config->methodVar   = 'f';               // RequestType=GET: the name of the method var.
$config->viewVar     = 't';               // RequestType=GET: the name of the view var.
$config->sessionVar  = 'yid';             // The session var name.

/* Set the allowed tags.  */
$config->allowedTags = new stdclass();
$config->allowedTags->front = '<p><span><h1><h2><h3><h4><h5><em><u><strong><br><ol><ul><li><img><a><b><font><hr><pre>';           // For front mode.
$config->allowedTags->admin = $config->allowedTags->front . '<div><table><td><th><tr><tbody><iframe><embed><style><header><nav>'; // For admin users.

/* Supported view types.*/
$config->views = ',html,json,xml,';

/* Init site config. */
$config->site = new stdclass();

/* Default theme. */
$config->site->theme = 'default';

/* Suported languags. */
$config->langs['zh-cn'] = '简体';
$config->langs['zh-tw'] = '繁体';

/* Supported charsets. */
$config->charsets['zh-cn']['utf-8'] = 'UTF-8';
$config->charsets['zh-cn']['gbk']   = 'GBK';
$config->charsets['zh-tw']['utf-8'] = 'UTF-8';
$config->charsets['zh-tw']['big5']  = 'BIG5';
$config->charsets['en']['utf-8']    = 'UTF-8';

/* Default params. */
$config->default = new stdclass();          
$config->default->view   = 'html';             // Default view.
$config->default->lang   = 'zh-cn';            // Default language.
$config->default->theme  = 'default';          // Default theme.
$config->default->module = 'index';            // Default module.
$config->default->method = 'index';            // Default method

/* Upload settings. */
$config->file = new stdclass();          
$config->file->dangers = 'php,jsp,py,rb,asp,'; // Dangerous file types.
$config->file->maxSize = 1024 * 1024;          // Max size allowed(Byte).

/* Database settings. */
$config->db = new stdclass();          
$config->db->persistant = false;               // Persistant connection or not.
$config->db->driver     = 'mysql';             // The driver of pdo, only mysql yet.
$config->db->encoding   = 'UTF8';              // The encoding of the database.
$config->db->strictMode = false;               // Turn off the strict mode.
$config->db->prefix     = 'yd_';               // The prefix of the table name.

/* Include my.php, domain.php and front or admin.php. */
$configRoot   = dirname(__FILE__) . DIRECTORY_SEPARATOR;
$myConfig     = $configRoot . 'my.php';
$domainConfig = $configRoot . 'domain.php';
$modeConfig   = $configRoot . RUN_MODE . '.php';

if(file_exists($myConfig))     include $myConfig;
if(file_exists($domainConfig)) include $domainConfig;
if(file_exists($modeConfig))   include $modeConfig;

/* The tables. */
define('TABLE_CONFIG',           $config->db->prefix . 'config');
define('TABLE_CATEGORY',         $config->db->prefix . 'category');
define('TABLE_ORDER',            $config->db->prefix . 'order');
define('TABLE_USER',             $config->db->prefix . 'user');
define('TABLE_STORE',            $config->db->prefix . 'store');
define('TABLE_SHELF',            $config->db->prefix . 'shelf');
define('TABLE_RELATION',         $config->db->prefix . 'relation');
define('TABLE_PRODUCT',          $config->db->prefix . 'product');
define('TABLE_CONTACT',          $config->db->prefix . 'contact');
define('TABLE_COMPANY',          $config->db->prefix . 'company');
define('TABLE_OAUTH',            $config->db->prefix . 'oauth');
define('TABLE_ORDER_REFUND',     $config->db->prefix . 'order_refund');
define('TABLE_ORDER_PRODUCT',    $config->db->prefix . 'order_product');
define('TABLE_BATCH',            $config->db->prefix . 'batch');
define('TABLE_BATCH_PRODUCT',    $config->db->prefix . 'batch_product');
define('TABLE_FILE',             $config->db->prefix . 'file');
define('TABLE_BALANCE',          $config->db->prefix . 'balance');
define('TABLE_DEPOSITOR',        $config->db->prefix . 'depositor');
define('TABLE_TRADE',            $config->db->prefix . 'trade');
define('TABLE_ACTION',           $config->db->prefix . 'action');
define('TABLE_HISTORY',          $config->db->prefix . 'history');
define('TABLE_GROUP',            $config->db->prefix . 'group');
define('TABLE_USERGROUP',        $config->db->prefix . 'usergroup');
define('TABLE_GROUPPRIV',        $config->db->prefix . 'grouppriv');
define('TABLE_INITIALBALANCE',   $config->db->prefix . 'initialbalance');
define('TABLE_PURCHASE_PRODUCT', $config->db->prefix . 'purchase_product');
define('TABLE_INVOCE',           $config->db->prefix . 'invoice');
define('TABLE_INVOCE_PRODUCT',   $config->db->prefix . 'invoice_product');

/* Include extension config files. */
$extConfigFiles = glob($configRoot . 'ext' . DIRECTORY_SEPARATOR . '*.php');
if($extConfigFiles) foreach($extConfigFiles as $extConfigFile) include $extConfigFile;

/* Include the cache file. */
$cacheConfigFile = dirname($configRoot) . DIRECTORY_SEPARATOR . 'tmp' . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR . 'config.php';
if(file_exists($cacheConfigFile)) include $cacheConfigFile;
